@@ -1,33 +1,15 @@
-/************************************************************
+/*
  * Copyright (c) 1994 by Silicon Graphics Computer Systems, Inc.
- *
- * Permission to use, copy, modify, and distribute this
- * software and its documentation for any purpose and without
- * fee is hereby granted, provided that the above copyright
- * notice appear in all copies and that both that copyright
- * notice and this permission notice appear in supporting
- * documentation, and that the name of Silicon Graphics not be
- * used in advertising or publicity pertaining to distribution
- * of the software without specific prior written permission.
- * Silicon Graphics makes no representation about the suitability
- * of this software for any purpose. It is provided "as is"
- * without any express or implied warranty.
- *
- * SILICON GRAPHICS DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS
- * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
- * AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL SILICON
- * GRAPHICS BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL
- * DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
- * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE
- * OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
- * THE USE OR PERFORMANCE OF THIS SOFTWARE.
- *
- ********************************************************/
+ * SPDX-License-Identifier: HPND
+ */
+
+#include "config.h"
 
 #include "xkbcomp-priv.h"
 #include "text.h"
 #include "expr.h"
 #include "include.h"
+#include "util-mem.h"
 
 typedef struct {
     enum merge_mode merge;
@@ -45,6 +27,7 @@ typedef struct {
 typedef struct {
     char *name;
     int errorCount;
+    unsigned int include_depth;
 
     xkb_keycode_t min_key_code;
     xkb_keycode_t max_key_code;
@@ -97,7 +80,7 @@ AddLedName(KeyNamesInfo *info, enum merge_mode merge, bool same_file,
     old = FindLedByName(info, new->name, &old_idx);
     if (old) {
         if (old_idx == new_idx) {
-            log_warn(info->ctx,
+            log_warn(info->ctx, XKB_LOG_MESSAGE_NO_ID,
                      "Multiple indicators named \"%s\"; "
                      "Identical definitions ignored\n",
                      xkb_atom_text(info->ctx, new->name));
@@ -107,7 +90,7 @@ AddLedName(KeyNamesInfo *info, enum merge_mode merge, bool same_file,
         if (report) {
             xkb_led_index_t use = (replace ? new_idx + 1 : old_idx + 1);
             xkb_led_index_t ignore = (replace ? old_idx + 1 : new_idx + 1);
-            log_warn(info->ctx,
+            log_warn(info->ctx, XKB_LOG_MESSAGE_NO_ID,
                      "Multiple indicators named %s; Using %d, ignoring %d\n",
                      xkb_atom_text(info->ctx, new->name), use, ignore);
         }
@@ -127,7 +110,8 @@ AddLedName(KeyNamesInfo *info, enum merge_mode merge, bool same_file,
         if (report) {
             const xkb_atom_t use = (replace ? new->name : old->name);
             const xkb_atom_t ignore = (replace ? old->name : new->name);
-            log_warn(info->ctx, "Multiple names for indicator %d; "
+            log_warn(info->ctx, XKB_LOG_MESSAGE_NO_ID,
+                     "Multiple names for indicator %d; "
                      "Using %s, ignoring %s\n", new_idx + 1,
                      xkb_atom_text(info->ctx, use),
                      xkb_atom_text(info->ctx, ignore));
@@ -152,10 +136,12 @@ ClearKeyNamesInfo(KeyNamesInfo *info)
 }
 
 static void
-InitKeyNamesInfo(KeyNamesInfo *info, struct xkb_context *ctx)
+InitKeyNamesInfo(KeyNamesInfo *info, struct xkb_context *ctx,
+                 unsigned int include_depth)
 {
     memset(info, 0, sizeof(*info));
     info->ctx = ctx;
+    info->include_depth = include_depth;
     info->min_key_code = XKB_KEYCODE_INVALID;
 #if XKB_KEYCODE_INVALID < XKB_KEYCODE_MAX
 #error "Hey, you can't be changing stuff like that."
@@ -184,6 +170,14 @@ AddKeyName(KeyNamesInfo *info, xkb_keycode_t kc, xkb_atom_t name,
 
     report = report && ((same_file && verbosity > 0) || verbosity > 7);
 
+    /* Check keycode is not too huge for our continuous array */
+    if (kc > XKB_KEYCODE_MAX_IMPL) {
+        log_err(info->ctx, XKB_LOG_MESSAGE_NO_ID,
+                "Keycode too big: must be < %#"PRIx32", got %#"PRIx32"; "
+                "Key ignored\n", XKB_KEYCODE_MAX_IMPL, kc);
+        return false;
+    }
+
     if (kc >= darray_size(info->key_names))
         darray_resize0(info->key_names, kc + 1);
 
@@ -198,7 +192,7 @@ AddKeyName(KeyNamesInfo *info, xkb_keycode_t kc, xkb_atom_t name,
 
         if (old_name == name) {
             if (report)
-                log_warn(info->ctx,
+                log_warn(info->ctx, XKB_LOG_MESSAGE_NO_ID,
                          "Multiple identical key name definitions; "
                          "Later occurrences of \"%s = %d\" ignored\n",
                          lname, kc);
@@ -206,14 +200,14 @@ AddKeyName(KeyNamesInfo *info, xkb_keycode_t kc, xkb_atom_t name,
         }
         else if (merge == MERGE_AUGMENT) {
             if (report)
-                log_warn(info->ctx,
+                log_warn(info->ctx, XKB_LOG_MESSAGE_NO_ID,
                          "Multiple names for keycode %d; "
                          "Using %s, ignoring %s\n", kc, lname, kname);
             return true;
         }
         else {
             if (report)
-                log_warn(info->ctx,
+                log_warn(info->ctx, XKB_LOG_MESSAGE_NO_ID,
                          "Multiple names for keycode %d; "
                          "Using %s, ignoring %s\n", kc, kname, lname);
             darray_item(info->key_names, kc) = XKB_ATOM_NONE;
@@ -228,13 +222,13 @@ AddKeyName(KeyNamesInfo *info, xkb_keycode_t kc, xkb_atom_t name,
         if (merge == MERGE_OVERRIDE) {
             darray_item(info->key_names, old_kc) = XKB_ATOM_NONE;
             if (report)
-                log_warn(info->ctx,
+                log_warn(info->ctx, XKB_WARNING_CONFLICTING_KEY_NAME,
                          "Key name %s assigned to multiple keys; "
                          "Using %d, ignoring %d\n", kname, kc, old_kc);
         }
         else {
             if (report)
-                log_vrb(info->ctx, 3,
+                log_vrb(info->ctx, 3, XKB_WARNING_CONFLICTING_KEY_NAME,
                         "Key name %s assigned to multiple keys; "
                         "Using %d, ignoring %d\n", kname, old_kc, kc);
             return true;
@@ -260,8 +254,7 @@ MergeIncludedKeycodes(KeyNamesInfo *into, KeyNamesInfo *from,
     }
 
     if (into->name == NULL) {
-        into->name = from->name;
-        from->name = NULL;
+        into->name = steal(&from->name);
     }
 
     /* Merge key names. */
@@ -275,7 +268,7 @@ MergeIncludedKeycodes(KeyNamesInfo *into, KeyNamesInfo *from,
         if (darray_size(into->key_names) < darray_size(from->key_names))
             darray_resize0(into->key_names, darray_size(from->key_names));
 
-        for (unsigned i = from->min_key_code; i <= from->max_key_code; i++) {
+        for (xkb_keycode_t i = from->min_key_code; i <= from->max_key_code; i++) {
             xkb_atom_t name = darray_item(from->key_names, i);
             if (name == XKB_ATOM_NONE)
                 continue;
@@ -334,9 +327,13 @@ HandleIncludeKeycodes(KeyNamesInfo *info, IncludeStmt *include)
 {
     KeyNamesInfo included;
 
-    InitKeyNamesInfo(&included, info->ctx);
-    included.name = include->stmt;
-    include->stmt = NULL;
+    if (ExceedsIncludeMaxDepth(info->ctx, info->include_depth)) {
+        info->errorCount += 10;
+        return false;
+    }
+
+    InitKeyNamesInfo(&included, info->ctx, 0 /* unused */);
+    included.name = steal(&include->stmt);
 
     for (IncludeStmt *stmt = include; stmt; stmt = stmt->next_incl) {
         KeyNamesInfo next_incl;
@@ -349,7 +346,7 @@ HandleIncludeKeycodes(KeyNamesInfo *info, IncludeStmt *include)
             return false;
         }
 
-        InitKeyNamesInfo(&next_incl, info->ctx);
+        InitKeyNamesInfo(&next_incl, info->ctx, info->include_depth + 1);
 
         HandleKeycodesFile(&next_incl, file, MERGE_OVERRIDE);
 
@@ -376,13 +373,14 @@ HandleKeycodeDef(KeyNamesInfo *info, KeycodeDef *stmt, enum merge_mode merge)
     }
 
     if (stmt->value < 0 || stmt->value > XKB_KEYCODE_MAX) {
-        log_err(info->ctx,
+        log_err(info->ctx, XKB_LOG_MESSAGE_NO_ID,
                 "Illegal keycode %lld: must be between 0..%u; "
                 "Key ignored\n", (long long) stmt->value, XKB_KEYCODE_MAX);
         return false;
     }
 
-    return AddKeyName(info, stmt->value, stmt->name, merge, false, true);
+    return AddKeyName(info, (xkb_keycode_t) stmt->value,
+                      stmt->name, merge, false, true);
 }
 
 static bool
@@ -394,6 +392,7 @@ HandleAliasDef(KeyNamesInfo *info, KeyAliasDef *def, enum merge_mode merge)
         if (old->alias == def->alias) {
             if (def->real == old->real) {
                 log_vrb(info->ctx, 1,
+                        XKB_WARNING_CONFLICTING_KEY_NAME,
                         "Alias of %s for %s declared more than once; "
                         "First definition ignored\n",
                         KeyNameText(info->ctx, def->alias),
@@ -405,7 +404,7 @@ HandleAliasDef(KeyNamesInfo *info, KeyAliasDef *def, enum merge_mode merge)
                 use = (merge == MERGE_AUGMENT ? old->real : def->real);
                 ignore = (merge == MERGE_AUGMENT ? def->real : old->real);
 
-                log_warn(info->ctx,
+                log_warn(info->ctx, XKB_WARNING_CONFLICTING_KEY_NAME,
                          "Multiple definitions for alias %s; "
                          "Using %s, ignoring %s\n",
                          KeyNameText(info->ctx, old->alias),
@@ -435,14 +434,16 @@ HandleKeyNameVar(KeyNamesInfo *info, VarDef *stmt)
         return false;
 
     if (elem) {
-        log_err(info->ctx, "Unknown element %s encountered; "
-                "Default for field %s ignored\n", elem, field);
+        log_err(info->ctx, XKB_ERROR_GLOBAL_DEFAULTS_WRONG_SCOPE,
+                "Cannot set global defaults for \"%s\" element; "
+                "Assignment to \"%s.%s\" ignored\n", elem, elem, field);
         return false;
     }
 
     if (!istreq(field, "minimum") && !istreq(field, "maximum")) {
-        log_err(info->ctx, "Unknown field encountered; "
-                "Assignment to field %s ignored\n", field);
+        log_err(info->ctx, XKB_LOG_MESSAGE_NO_ID,
+                "Unknown field encountered; "
+                "Assignment to field \"%s\" ignored\n", field);
         return false;
     }
 
@@ -454,27 +455,25 @@ static bool
 HandleLedNameDef(KeyNamesInfo *info, LedNameDef *def,
                  enum merge_mode merge)
 {
-    LedNameInfo ledi;
-    xkb_atom_t name;
-
     if (def->ndx < 1 || def->ndx > XKB_MAX_LEDS) {
         info->errorCount++;
-        log_err(info->ctx,
-                "Illegal indicator index (%d) specified; must be between 1 .. %d; "
-                "Ignored\n", def->ndx, XKB_MAX_LEDS);
+        log_err(info->ctx, XKB_LOG_MESSAGE_NO_ID,
+                "Illegal indicator index (%"PRId64") specified; "
+                "must be between 1 .. %u; Ignored\n", def->ndx, XKB_MAX_LEDS);
         return false;
     }
 
+    xkb_atom_t name = XKB_ATOM_NONE;
     if (!ExprResolveString(info->ctx, def->name, &name)) {
         char buf[20];
-        snprintf(buf, sizeof(buf), "%u", def->ndx);
+        snprintf(buf, sizeof(buf), "%"PRId64, def->ndx);
         info->errorCount++;
-        return ReportBadType(info->ctx, "indicator", "name", buf, "string");
+        return ReportBadType(info->ctx, XKB_ERROR_WRONG_FIELD_TYPE,
+                             "indicator", "name", buf, "string");
     }
 
-    ledi.merge = merge;
-    ledi.name = name;
-    return AddLedName(info, merge, true, &ledi, def->ndx - 1);
+    LedNameInfo ledi = {.merge = merge, .name = name};
+    return AddLedName(info, merge, true, &ledi, (xkb_led_index_t) def->ndx - 1);
 }
 
 static void
@@ -503,7 +502,7 @@ HandleKeycodesFile(KeyNamesInfo *info, XkbFile *file, enum merge_mode merge)
             ok = HandleLedNameDef(info, (LedNameDef *) stmt, merge);
             break;
         default:
-            log_err(info->ctx,
+            log_err(info->ctx, XKB_LOG_MESSAGE_NO_ID,
                     "Keycode files may define key and indicator names only; "
                     "Ignoring %s\n", stmt_type_to_string(stmt->type));
             ok = false;
@@ -514,7 +513,8 @@ HandleKeycodesFile(KeyNamesInfo *info, XkbFile *file, enum merge_mode merge)
             info->errorCount++;
 
         if (info->errorCount > 10) {
-            log_err(info->ctx, "Abandoning keycodes file \"%s\"\n",
+            log_err(info->ctx, XKB_LOG_MESSAGE_NO_ID,
+                    "Abandoning keycodes file \"%s\"\n",
                     file->name);
             break;
         }
@@ -569,6 +569,7 @@ CopyKeyAliasesToKeymap(struct xkb_keymap *keymap, KeyNamesInfo *info)
         /* Check that ->real is a key. */
         if (!XkbKeyByName(keymap, alias->real, false)) {
             log_vrb(info->ctx, 5,
+                    XKB_WARNING_UNDEFINED_KEYCODE,
                     "Attempt to alias %s to non-existent key %s; Ignored\n",
                     KeyNameText(info->ctx, alias->alias),
                     KeyNameText(info->ctx, alias->real));
@@ -579,6 +580,7 @@ CopyKeyAliasesToKeymap(struct xkb_keymap *keymap, KeyNamesInfo *info)
         /* Check that ->alias is not a key. */
         if (XkbKeyByName(keymap, alias->alias, false)) {
             log_vrb(info->ctx, 5,
+                    XKB_WARNING_ILLEGAL_KEYCODE_ALIAS,
                     "Attempt to create alias with the name of a real key; "
                     "Alias \"%s = %s\" ignored\n",
                     KeyNameText(info->ctx, alias->alias),
@@ -596,14 +598,14 @@ CopyKeyAliasesToKeymap(struct xkb_keymap *keymap, KeyNamesInfo *info)
         key_aliases = calloc(num_key_aliases, sizeof(*key_aliases));
         if (!key_aliases)
             return false;
-    }
 
-    i = 0;
-    darray_foreach(alias, info->aliases) {
-        if (alias->real != XKB_ATOM_NONE) {
-            key_aliases[i].alias = alias->alias;
-            key_aliases[i].real = alias->real;
-            i++;
+        i = 0;
+        darray_foreach(alias, info->aliases) {
+            if (alias->real != XKB_ATOM_NONE) {
+                key_aliases[i].alias = alias->alias;
+                key_aliases[i].real = alias->real;
+                i++;
+            }
         }
     }
 
@@ -650,7 +652,7 @@ CompileKeycodes(XkbFile *file, struct xkb_keymap *keymap,
 {
     KeyNamesInfo info;
 
-    InitKeyNamesInfo(&info, keymap->ctx);
+    InitKeyNamesInfo(&info, keymap->ctx, 0);
 
     HandleKeycodesFile(&info, file, merge);
     if (info.errorCount != 0)
